"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const lodash_clonedeep_1 = tslib_1.__importDefault(require("lodash.clonedeep"));
const to_case_1 = tslib_1.__importDefault(require("to-case"));
const utils_1 = require("../utils");
const Collection_1 = tslib_1.__importDefault(require("./Collection"));
const EventEmitter_1 = tslib_1.__importDefault(require("../EventEmitter"));
class Model extends EventEmitter_1.default {
    constructor(props = {}) {
        super();
        this.loading = false;
        this.submittableProps = [];
        this.performSafeRequestGraphql = utils_1.performSafeRequestGraphql;
        this.performSafeRequestREST = utils_1.performSafeRequestREST;
        this._exists = false;
        Object.assign(this, this.defaults(), props);
        Object.assign(this, props);
        if (typeof this.hydrateProps === 'function') {
            console.warn('Deprecated: Use property variables or constructor instead. `hydrateProps` will be removed in the next versions.');
            this.hydrateProps(props);
        }
    }
    get doRequest() {
        return utils_1.config().graphql ? utils_1.performSafeRequestGraphql : utils_1.performSafeRequestREST;
    }
    get className() {
        return this.constructor.name;
    }
    get displayClassName() {
        return this.className.toLowerCase();
    }
    get defaultMethod() {
        return 'post';
    }
    exists() {
        return this._exists;
    }
    defaults() {
        return {};
    }
    hydrate(item) {
        return new this.constructor(item);
    }
    getMethod(_opts) {
        return null;
    }
    toCollection(skipEmpty = false) {
        return (new Collection_1.default(this)).filter(i => !!i || !skipEmpty);
    }
    toArray(skipEmpty = false) {
        return this.toCollection(skipEmpty).all();
    }
    getClassName() {
        return this.constructor.name;
    }
    getInputFields() {
        return utils_1.getSchemaTypeFields(`${this.getClassName()}Input`);
    }
    getUpdateVariables() {
        return utils_1.stripObject(this.toCollection()
            .only(this.getInputFields())
            .map(field => {
            if (Array.isArray(field)) {
                return field.map(model => ((model instanceof Model) ? model.getUpdateVariables() : model));
            }
            if (field instanceof Model) {
                return field.getUpdateVariables();
            }
            return field;
        })
            .all());
    }
    toSubmittable(props = this.submittableProps) {
        if (!props)
            return [];
        if (Array.isArray(props) && !props.length)
            return [];
        return this.toCollection().only(props).all();
    }
    save() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.exists() ? this.update() : this.create();
        });
    }
    create(params, mutation) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const resolvedRequest = this.resolveRequest(mutation, this.createMutation);
            const resolvedParams = this.resolveParams(params);
            if (utils_1.config().graphql) {
                return this.hydrate(yield this.mutate(resolvedRequest, resolvedParams));
            }
            return this.hydrate(yield this.mutate(resolvedRequest, resolvedParams));
        });
    }
    update(params, mutation) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const resolvedRequest = this.resolveRequest(mutation, this.updateMutation);
            const resolvedParams = this.resolveParams(params);
            if (utils_1.config().graphql) {
                return this.hydrate(yield this.mutate(resolvedRequest, resolvedParams));
            }
            return this.hydrate(yield this.mutate(resolvedRequest, resolvedParams));
        });
    }
    delete(params, mutation) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const resolvedRequest = this.resolveRequest(mutation, this.deleteMutation);
            const forget = result => this.markNotExists() && result;
            if (utils_1.config().graphql) {
                return this.mutate(resolvedRequest, params || {
                    uuid: this.uuid,
                }).then(forget);
            }
            return this.mutate(resolvedRequest, params || {
                id: this.id,
            }, 'delete').then(forget);
        });
    }
    mutate(mutationOrUrl, params, method = this.defaultMethod) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (utils_1.config().graphql) {
                return this.beforeMutate()
                    .then(this.performSafeRequestGraphql.bind(this, mutationOrUrl, params, null))
                    .finally(this.afterMutate.bind(this))
                    .finally(this.markExists.bind(this));
            }
            const resolvedUrl = yield utils_1.getUrl({ method, url: mutationOrUrl, params });
            const resolvedMethod = yield this.getMethod({ method, url: resolvedUrl, params });
            return this.beforeMutate()
                .then(this.performSafeRequestREST.bind(this, resolvedUrl, params, resolvedMethod || method, null))
                .finally(this.afterMutate.bind(this))
                .finally(this.markExists.bind(this));
        });
    }
    clone() {
        return lodash_clonedeep_1.default(this).markNotExists();
    }
    markExists() {
        this._exists = true;
        return this;
    }
    markNotExists() {
        this._exists = false;
        return this;
    }
    beforeMutate() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.loading = true;
            return this;
        });
    }
    afterMutate() {
        this.loading = false;
        return this;
    }
    resolveRequest(request, defaultRequest) {
        if (request)
            return request;
        return typeof defaultRequest === 'function' ? defaultRequest() : defaultRequest;
    }
    resolveParams(params) {
        if (utils_1.config().graphql) {
            const model = this.getUpdateVariables();
            return {
                [to_case_1.default.camel(this.className)]: model,
            };
        }
        return params;
    }
}
exports.default = Model;
//# sourceMappingURL=Model.js.map