"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const graphql_1 = require("graphql");
const Registry_1 = tslib_1.__importDefault(require("./Registry"));
const UnexpectedException_1 = tslib_1.__importDefault(require("./models/Exceptions/UnexpectedException"));
exports.defaultRESTHeaders = {
    'Accept': 'application/json',
    'Content-Type': 'application/json;charset=UTF-8',
};
function getApollo() {
    return Registry_1.default.getInstance().get('Vue').$apollo;
}
exports.getApollo = getApollo;
function camelToKebab(input) {
    return input.replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase();
}
exports.camelToKebab = camelToKebab;
function queryParams(params, camelToKebabActive = true) {
    const str = [];
    for (const paramsKey in params)
        if (params.hasOwnProperty(paramsKey) && (params[paramsKey] || params[paramsKey] === 0)) {
            const normalized = camelToKebabActive ? camelToKebab(encodeURIComponent(paramsKey)) : encodeURIComponent(paramsKey);
            str.push(`${normalized}=${encodeURIComponent(params[paramsKey])}`);
        }
    return str.join('&');
}
exports.queryParams = queryParams;
function status(res) {
    const response = res.hasOwnProperty('status') ? res : res.response;
    if (response.status && response.status >= 200 && response.status < 300) {
        return Promise.resolve(response);
    }
    else {
        if (response.data && response.data.errors) {
            const errors = (response.data.errors || []).map((err) => err.defaultMessage).join(', ');
            return Promise.reject(new Error(errors));
        }
        return Promise.reject(new Error(response.data.message));
    }
}
exports.status = status;
function json(response) {
    if (response && response.data) {
        return response.data;
    }
    else {
        return null;
    }
}
exports.json = json;
function performSafeRequestREST(url, params = {}, method = 'get', opts = {}) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        let fullUrl = url;
        let body = {};
        if (method === 'get') {
            const queryEscaped = queryParams(params);
            if (queryEscaped) {
                fullUrl = `${fullUrl}?${queryEscaped}`;
            }
        }
        else {
            body = params;
        }
        return fetch(fullUrl, Object.assign({ method: method.toLowerCase(), headers: exports.defaultRESTHeaders, body: JSON.stringify(body) }, opts)).then(status).then(json);
    });
}
exports.performSafeRequestREST = performSafeRequestREST;
function performGqlQuery(query, variables, queryName, subscribeToMore) {
    return new Promise((result, error) => getApollo().addSmartQuery(queryName, {
        manual: true,
        query,
        subscribeToMore,
        variables,
        result,
        error,
    }));
}
exports.performGqlQuery = performGqlQuery;
function performGqlMutation(mutation, variables) {
    return new Promise((resolve, reject) => getApollo().mutate({
        update(_cache, result) {
            return resolve(result);
        },
        mutation,
        variables,
    }).catch(reject));
}
exports.performGqlMutation = performGqlMutation;
function stripTypename(obj) {
    return JSON.parse(JSON.stringify(obj, (k, v) => (k === '__typename' ? undefined : v)));
}
exports.stripTypename = stripTypename;
function performSafeRequestGraphql(query, variables = {}, subscriptions = []) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const queryName = query.definitions.map(def => def.name).find(def => def.kind === 'Name').value;
        const operation = query.definitions.find(def => def.kind === 'OperationDefinition').operation === 'query'
            ? performGqlQuery.bind(null, query, stripTypename(variables), queryName, subscriptions)
            : performGqlMutation.bind(null, query, stripTypename(variables));
        return operation().then(({ data }) => data[queryName]);
    });
}
exports.performSafeRequestGraphql = performSafeRequestGraphql;
function registryGet(key) {
    return Registry_1.default.getInstance().get(key);
}
exports.registryGet = registryGet;
function config() {
    return registryGet('Config');
}
exports.config = config;
function getParsedSchema() {
    const schema = config().schema;
    if (!schema) {
        throw new UnexpectedException_1.default('Configuration error: \'schema\' must be passed as a config key, e.g\n\nimport schema from \'raw-loader!@/../schema.graphql\';\n\n//...\n\nVue.use(VueOOP, {\n  //...,\n  schema,\n})\n\n;');
    }
    return schema;
}
exports.getParsedSchema = getParsedSchema;
function getSchemaTypeFields(typeName) {
    return getParsedSchema()
        .definitions
        .find(def => (def.name || {}).value === typeName)
        .fields
        .map(f => f.name.value);
}
exports.getSchemaTypeFields = getSchemaTypeFields;
function getSchemaMutation(mutationName) {
    return getParsedSchema()
        .definitions
        .find(def => (def.name || {}).value === 'Mutation')
        .fields
        .find(def => (def.name || {}).value === mutationName);
}
exports.getSchemaMutation = getSchemaMutation;
function getSchemaQuery(queryName) {
    return getParsedSchema()
        .definitions
        .find(def => (def.name || {}).value === 'Query')
        .fields
        .find(def => (def.name || {}).value === queryName);
}
exports.getSchemaQuery = getSchemaQuery;
function getUrl(_opts) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { url, params } = _opts;
        let resolvedUrl = url;
        if (typeof url === 'function') {
            resolvedUrl = yield url();
        }
        else {
            resolvedUrl = resolvedUrl.replace(/:([^\s\/?&]+)/gi, (_, m) => {
                const param = params[m];
                const hasParam = param !== undefined;
                if (hasParam) {
                    delete params[m];
                    return param;
                }
                return m;
            });
        }
        return resolvedUrl;
    });
}
exports.getUrl = getUrl;
function stripObject(obj) {
    return JSON.parse(JSON.stringify(obj, (k, v) => (k === 'loading' ? undefined : v)));
}
exports.stripObject = stripObject;
function fetchIntrospectionSchema(url) {
    const body = JSON.stringify({ query: graphql_1.getIntrospectionQuery() });
    return fetch(url, {
        method: 'POST',
        headers: { 'Content-Type': 'application/json' },
        body
    })
        .then(res => res.json())
        .then(res => res.data);
}
exports.fetchIntrospectionSchema = fetchIntrospectionSchema;
//# sourceMappingURL=utils.js.map